// **********************************************************************
//
// Copyright (c) 2003-2008 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

import Demo.*;

class Client extends Ice.Application
{
    class PingThread extends Thread
    {
	PingThread(PingPrx ping, int rep)
	{
	    _ping = PingPrxHelper.uncheckedCast(ping.ice_connectionId(Ice.Util.generateUUID()));
	    _ping.ice_ping();
	    _rep = rep;
	}

	public void
	run()
	{
	    System.out.println("pinging server " + _rep + " times (this may take a while)");
	    byte[] data = new byte[1024];
	    for(int i = 0; i < _rep; i++)
	    {
		byte[] r = _ping.test(data);
	    }
	}

	PingPrx _ping;
	int _rep;
    }

    public void
    usage()
    {
	System.err.println("usage: --threads <t> --rep <r>");
    }

    public int
    run(String[] args)
    {
	int rep = 10000;
	int threads = 1;
        for(int i = 0; i < args.length; ++i)
        {
            if(args[i].equals("--threads"))
            {
                ++i;
                if(i >= args.length)
                {
                    usage();
                    return 1;
                }
                threads = Integer.parseInt(args[i]);
            }
            else if(args[i].equals("--rep"))
            {
                ++i;
                if(i >= args.length)
                {
                    usage();
                    return 1;
                }
                rep = Integer.parseInt(args[i]);
            }
            else
            {
                usage();
                return 1;
            }
        }

        PingPrx ping = PingPrxHelper.checkedCast(communicator().propertyToProxy("Ping.Proxy"));
        if(ping == null)
        {
            System.err.println("invalid proxy");
            return 1;
        }

        //
        // A method needs to be invoked thousands of times before the JIT compiler
        // will convert it to native code. To ensure an accurate latency measurement,
        // we need to "warm up" the JIT compiler.
        //
        {
            final int repetitions = 20000;
            System.out.print("warming up the JIT compiler...");
            System.out.flush();
            for(int i = 0; i < repetitions; i++)
            {
                ping.ice_ping();
            }
            System.out.println(" ok");
        }

	PingThread t[] = new PingThread[threads];
	for(int i = 0; i < t.length; ++i)
	{
	    t[i] = new PingThread(ping, rep);
	}
        long tv1 = System.currentTimeMillis();
	for(int i = 0; i < t.length; ++i)
	{
	    t[i].start();
	}
	for(int i = 0; i < t.length; ++i)
	{
	    try
	    {
		t[i].join();
	    }
	    catch(InterruptedException e)
	    {
	    }
	}

        long tv2 = System.currentTimeMillis();
        double total = (double)(tv2 - tv1);
        double perPing = total / (rep * threads);

        System.out.println("time for " + rep * threads + " pings: " + total + "ms");
        System.out.println("time per ping: " + perPing + "ms");

        return 0;
    }

    public static void
    main(String[] args)
    {
        Client app = new Client();
        int status = app.main("Client", args, "config.client");
        System.exit(status);
    }
}
